# -*- coding: iso-8859-15 -*-
#
# AD ADMIN
#
#
# ******************************************
#    DO NOT MAKE ANY CHANGES TO THIS FILE
# ******************************************
#


import os, sys, default, new, string, re, time, host, bf2
import ad_framework_settings as adfs
import ad_framework as adf

# File handler
loggingFile = ""

# Name of the file in which the admincommandlogs are stored
loggingFileName = "admin/ad_framework/logs/ad_admin.txt"

# Variable all the possible admin commands will be stored in
adminCommands = ""

# Maplist (the active maplist on the server)
mapList = []

# If true, the script uses cd key hashes to verify administrators. If false, it skips those checks, so one is able to test the script on a local server
isDedicated = True


# Initialise the module
def init():
	adf.Debug("Initializing AD Admin")

	global sys, isDedicated, adminCommands
	host.registerHandler('PlayerKilled', onPlayerKilled)
	host.registerHandler('PlayerDeath', onPlayerDeath)
	host.registerHandler('PlayerConnect', onPlayerConnect)
	host.registerHandler('ChatMessage', onChatMessage, 1)
	host.registerHandler('ExitVehicle', onExitVehicle)

	# Check to see if the server is dedicated or not
	if int(host.rcon_invoke("sv.internet").strip()) == 1:
		isDedicated = True
	else:
		isDedicated = False

	# Fill the dictionary with all the available commands
	adminCommands = {
		"ab":			PythonBalance,
		"admins":		TextAdmins,
		"b":			PlayerBan,
		"ban":			PlayerBan,
		"change":		MapChange,
		"debug":		PythonDebug,
		"fly":			PlayerFly,
		"hash":			PlayerHash,
		"help":			TextHelp,
		"init":			PythonInit,
		"k":			PlayerKick,
		"kick":			PlayerKick,
		"kill":			PlayerKill,
		"mumble":		TextMumble,
		"r":			PlayerReport,
		"reload":		MapReload,
		"remove":		MapRemove,
		"report":		PlayerReport,
		"resign":		PlayerResign,
		"restart":		MapRestart,
		"rules":		TextRules,
		"runnext":		MapRunNext,
		"s":			TextSay,
		"save":			MapListSave,
		"say":			TextSay,
		"sayteam":		TextSayTeam,
		"setnext":		MapSetNext,
		"shownext":		MapShowNext,
		"ss":			PlayerScreenshot,
		"st":			TextSayTeam,
		"stopserver":	ServerStop,
		"swapteams":	PlayerSwapTeams,
		"switch":		PlayerSwitch,
		"tb":			PlayerBanTemp,
		#"ub":			PlayerUnban,
		#"unban":		PlayerUnban,
		"version":		TextVersion,
		"vote":			ServerVote,
		"w":			PlayerWarn,
		"warn":			PlayerWarn
	}


#
# ======================= ADMIN COMMANDS =======================
#


# Show the next map
def MapShowNext(cmd, args, p):
	try:
		fMapName = ""
		fMapType = ""
		fMapSize = ""

		ConstructMaplist()
		nextMapID = host.rcon_invoke("admin.nextLevel").strip()
		for nextMap in mapList:
			map = nextMap.split("|")
			if map[0] == nextMapID:
				fMapName = FormatMapProperties(map[1])
				fMapType = FormatMapProperties(map[2])
				fMapSize = FormatMapProperties(map[3])

				adf.PersonalMessage("C1001Next map in rotation is: " + fMapName + " (" + fMapType + ", " + fMapSize + ")", p)
				break
	except:
		adf.Debug("Exception in MapShowNext()")


# Change the next map and run it
def MapChange(cmd, args, p):
	try:
		if not MapSetNext(cmd, args, p) == False:
			AdminPM("C1001Changing map", p)
			host.rcon_invoke('game.sayall "C1001Running next map..."')
			host.rcon_invoke('admin.runNextLevel')
			Log(cmd, p.getName(), "", "(see previous entry)")
	except:
		adf.Debug("Exception in MapChange()")


# Save the current maplist
def MapListSave(cmd, args, p):
	try:
		host.rcon_invoke("maplist.save")
		AdminPM("C1001The current maplist has been saved", p)
		Log(cmd, p.getName(), "", "")
	except:
		adf.Debug("Exception in MapListSave()")


# Remove a map from the maplist
def MapRemove(cmd, args, p):
	try:
		fMapName = ""
		fMapType = ""
		fMapSize = ""

		if len(args) < 1 or args[0].isdigit() == False:
			adf.PersonalMessage("C1001Please specify a mapid", p)
			return False

		# Setting a next map using a mapid
		ConstructMaplist()
		matches = 0
		for nextMap in mapList:
			map = nextMap.split("|")
			mapID = map[0]
			mapName = map[1]
			mapType = map[2]
			mapSize = map[3]

			# If the ID's match, we have a winner..
			if map[0] == args[0]:
				matches = 1

				# Remove the map
				host.rcon_invoke("maplist.remove %s" % str(mapID))

				# Format map properties
				fMapName = FormatMapProperties(mapName)
				fMapType = FormatMapProperties(mapType)
				fMapSize = FormatMapProperties(mapSize)

				# Display removal text
				AdminPM("C1001The following map has been removed from the rotation: " + fMapName + " (" + fMapType + ", " + fMapSize + ")", p)
				Log(cmd, p.getName(), "", fMapName + " (" + fMapType + ", " + fMapSize + ")")
				break

		# If there aren't any matches, return (stop execution)
		if matches == 0:
			adf.PersonalMessage("C1001Sorry, no matching map found for mapID: " + args[0], p)
			return False
	except:
		adf.Debug("Exception in MapRemove()")


# Set the next map
def MapSetNext(cmd, args, p):
	try:
		fMapName = ""
		fMapType = ""
		fMapSize = ""

		if len(args) < 1:
			adf.PersonalMessage("C1001Please specify a mapname or mapid", p)
			return False

		# Setting a next map using a mapid
		elif args[0].isdigit() == True:
			ConstructMaplist()
			matches = 0
			for nextMap in mapList:
				map = nextMap.split("|")
				mapID = map[0]
				mapName = map[1]
				mapType = map[2]
				mapSize = map[3]

				# If the ID's match, we have a winner..
				if map[0] == args[0]:
					matches = 1
					host.rcon_invoke("admin.nextLevel %s" % str(mapID))

					# Format map properties
					fMapName = FormatMapProperties(map[1])
					fMapType = FormatMapProperties(map[2])
					fMapSize = FormatMapProperties(map[3])

					# Display text
					AdminPM("C1001Next map in rotation set to: " + fMapName + " (" + fMapType + ", " + fMapSize + ")", p)
					Log(cmd, p.getName(), "", fMapName + " (" + fMapType + ", " + fMapSize + ")")
					break

			# If there aren't any matches, return (stop execution)
			if matches == 0:
				adf.PersonalMessage("C1001Sorry, no matching map found for mapID: " + args[0], p)
				return False

		# Setting a next map using the mapname, gamemode and mapsize (search through server's maplist, if fail, then search through adm_mapListAll)
		else:
			if len(args) < 3:
				adf.PersonalMessage("C1001Please specify a mapname, gamemode and layer", p)
				return False

			if args[2] == "16" or args[2] == "32" or args[2] == "64":
				adf.PersonalMessage("C1001You can no longer use 16/32/64, use inf, alt or std instead", p)
				return False

			ConstructMaplist()
			matches = 0
			for nextMap in mapList:
				map = nextMap.split("|")
				# [0] = mapID
				# [1] = mapname
				# [2] = maptype
				# [3] = mapsize

				# Does the entered mapname, gamemode and layer exist in the server's maplist?
				if map[1].lower().find(args[0].lower()) != -1 and map[2].lower().find(args[1].lower()) != -1 and map[3].lower().find(args[2].lower()) != -1:
					adf.Debug("Mapname, gamemode and layer found in maplist!")
					matches += 1
					mapID = map[0]
					mapName = map[1]
					mapType = map[2]
					mapSize = map[3]
					break

			# If no matches are found in the server's maplist, try to find it in adm_mapListAll
			if matches == 0:
				for nextMap in adfs.adm_mapListAll:
					map = nextMap.split("|")
					# [0] = mapname
					# [1] = maptype
					# [2] = mapsize

					# Search for the map in adm_mapListAll (mapname, gamemode, mapsize)
					if map[0].lower().find(args[0].lower()) != -1 and map[1].lower().find(args[1].lower()) != -1 and map[2].lower().find(args[2].lower()) != -1:
						adf.Debug("Mapname, gamemode and layer found in adm_mapListAll!")
						matches += 1
						break

				adf.Debug("matches after second search: " + str(matches))

				# Map found, append it to the current list
				if matches == 1:
					# Save map info, so we can add the map to the server's maplist later
					mapID = len(mapList)
					mapName = map[0]
					mapType = map[1]
					mapSize = map[2]

					# Format map properties
					fMapName = FormatMapProperties(mapName)
					fMapType = FormatMapProperties(mapType)
					fMapSize = FormatMapProperties(mapSize)

					# Translate mapsizes, so we add the correct ones
					if mapSize == "inf":
						mapSize = "16"
					elif mapSize == "alt":
						mapSize = "32"
					elif mapSize == "std":
						mapSize = "64"

					adf.Debug("Trying RCON commands " + str(mapName) + ", " + str(mapType) + ", " + str(mapSize))

					# Add the map
					host.rcon_invoke("maplist.append %s %s %s" % (mapName, mapType, mapSize))

					adf.Debug("Trying RCON commands... say: " + str(fMapName) + ", " + str(fMapType) + ", " + str(fMapSize))

					# Display text
					AdminPM("C1001The following map has been added to rotation: " + fMapName + " (" + fMapType + ", " + fMapSize + ")", p)

			adf.Debug("Done searching")

			# If there still aren't any matches, return (stop execution)
			if matches == 0:
				adf.PersonalMessage("C1001Sorry, no matching map found for: " + args[0] + " " + args[1] + " " + args[2], p)
				return False
			else:
				fMapName = FormatMapProperties(mapName)
				fMapType = FormatMapProperties(mapType)
				fMapSize = FormatMapProperties(mapSize)

				adf.Debug("Going to set the next map")
				host.rcon_invoke("admin.nextLevel %s" % str(mapID))

				AdminPM("C1001Next map in rotation set to: " + fMapName + " (" + fMapType + ", " + fMapSize + ")", p)
				Log(cmd, p.getName(), "", fMapName + " (" + fMapType + ", " + fMapSize + ")")
	except:
		adf.Debug("Exception in MapSetNext()")
		return False


# Run the next map
def MapRunNext(cmd, args, p):
	try:
		AdminPM("C1001Running next map", p)
		host.rcon_invoke('game.sayall "C1001Running next map..."')
		host.rcon_invoke('admin.runNextLevel')
		Log(cmd, p.getName(), "", "")
	except:
		adf.Debug("Exception in MapRunNext()")


# Restart the current map
def MapRestart(cmd, args, p):
	try:
		AdminPM("C1001Restarting map", p)
		host.rcon_invoke('game.sayall "C1001Restarting map..."')
		host.rcon_invoke('admin.restartMap')
		Log(cmd, p.getName(), "", "")
	except:
		adf.Debug("Exception in MapRestart()")


# Reload the current map (this reloads the entire map)
def MapReload(cmd, args, p):
	try:
		currentMapID = host.rcon_invoke("admin.currentLevel").strip()
		host.rcon_invoke("admin.nextLevel %s" % str(currentMapID))
		AdminPM("C1001Reloading map", p)
		host.rcon_invoke('game.sayall "C1001Reloading map..."')
		host.rcon_invoke('admin.runNextLevel')
		Log(cmd, p.getName(), "", "")
	except:
		adf.Debug("Exception in MapReload()")


def TextAdmins(cmd, args, p):
	try:
		adf.PersonalMessage("C1001The following admins are online:", p)

		admins = ""
		i = 0
		j = 0
		for	pl in bf2.playerManager.getPlayers():
			try:
				if pl.hash == "":
					pass
			except:
				adf.Debug("Playerhash-variable isn't set, player isn't inited, initiating now")
				PlayerInit(pl)

			if pl.hash == "":
				tHash =	adf.GetPlayerHash(pl.getName())
				if tHash ==	False:
					adf.Debug("Hash	not	found of " + pl.getName())
				else:
					adf.Debug("Hash	found, setting it now")
					pl.hash = tHash

			if pl.hash != "" and adfs.adm_adminHashes.has_key(pl.hash):
				i += 1
				j += 1

				if admins != "":
					admins += ", "
				admins += pl.getName()

				if i == 5:
					adf.PersonalMessage("C1001" + admins, p)
					i = 0
					admins = ""
		if i > 0:
			adf.PersonalMessage("C1001" + admins, p)

		if j == 0:
			adf.PersonalMessage("C1001none", p)
	except:
			adf.Debug("Exception in	TextAdmins()")


# Display text (global)
def TextSay(cmd, args, p):
	try:
		if len(args) < 1:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a message", p)
		else:
			i = 0
			message = ""
			while i < len(args):
				message += str(args[i] + " ")
				i += 1

			# Find a predefined reason
			message = FindReason(message)

			host.rcon_invoke('game.sayall "3%s"' % message)
			Log(cmd, p.getName(), "", message)
	except:
		adf.Debug("Exception in TextSay()")


# Display text (global)
def TextSayTeam(cmd, args, p):
	try:
		if len(args) < 2 or (args[0].lower() != "us" and args[0].lower() != "them"):
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a team [us/them] and a message", p)
		else:
			# Set the team
			if args[0].lower() == "us":
				team = p.getTeam()
			elif args[0].lower() == "them":
				if p.getTeam() == 1:
					team = 2
				else:
					team = 1

			# Form the text to display
			i = 1
			message = ""
			while i < len(args):
				message += str(args[i] + " ")
				i += 1

			# Find a predefined reason
			message = FindReason(message)
			
			host.rcon_invoke('game.sayTeam %s "3Team: %s"' % (team, message))
			Log(cmd, p.getName(), "team " + str(team), message)
	except:
		adf.Debug("Exception in TextSayTeam()")


# Display helpmessages
def TextHelp(cmd, args, p):
	global adminCommands

	try:
		adf.PersonalMessage("C1001Available commands:", p)
		i = 0
		keys = ""
		for key in adminCommands:
			i += 1
			if keys != "":
				keys += ", "
			keys += adfs.adm_commandSymbol + key

			if i == 8:
				adf.PersonalMessage("C1001" + keys, p)
				keys = ""
				i = 0

		# Display the remaining commands, if there are any
		if i > 0:
			adf.PersonalMessage("C1001" + keys, p)
		Log(cmd, p.getName(), "", "")
	except:
		adf.Debug("Exception in TextHelp()")


# Display the versionnumber of the AD Framework
def TextVersion(cmd, args, p):
	try:
		adf.PersonalMessage("C1001This server is running version " + adfs.version + " of the AD Framework", p)

		msg = ""
		if adfs.smb_enabled == True:
			msg += ", AD Smart Balance"
		if adfs.log_enabled == True:
			msg += ", AD Logs"
		if adfs.ann_enabled == True:
			msg += ", AD Announcer"

		adf.PersonalMessage("C1001Enabled modules: AD Admin" + msg, p)
	except:
		adf.Debug("Exception in TextHelp()")


# Display the Rules of the Server
def TextRules(cmd, args, p):
	try:
		if adfs.adm_rulesEnabled == True:
			for r in adfs.adm_rules:
				adf.PersonalMessage(r, p)
	except:
		adf.Debug("Exception in TextRules()")


# Display the "Mumble-message" as defined in the settings
def TextMumble(cmd, args, p):
	try:
		adf.PersonalMessage(adfs.adm_mumble, p)
	except:
		adf.Debug("Exception in TextMumble()")


# Resign a player
def PlayerResign(cmd, args, p):
	try:
		if len(args) < 2:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name and a reason", p)
		else:
			i = 1
			reason = ""
			while i < len(args):
				reason += str(args[i] + " ")
				i += 1

			foundPlayer = adf.FindPlayer(args[0])
			if foundPlayer == "none":
				adf.PersonalMessage("C1001Sorry, no matching player found for '" + args[0] + "'", p)
			elif foundPlayer == "more":
				adf.PersonalMessage("C1001Multiple players found that match '" + args[0] + "'", p)
			else:
				i = 1
				reason = ""
				while i < len(args):
					reason += str(args[i] + " ")
					i += 1

				# Find a predefined reason
				reason = FindReason(reason)

				# Resign the player
				AdminPM("C1001" + foundPlayer.getName() + " has been resigned, " + reason, p)
				host.rcon_invoke('game.sayall "C1001RESIGNING PLAYER %s, %s"' % (foundPlayer.getName(), reason))
				#bf2.gameLogic.sendGameEvent(foundPlayer, 13, 1)	# Blurs the screen of the resigned player for a second
				if foundPlayer.getTeam() == 1:
					foundPlayer.setTeam(2)
					foundPlayer.setTeam(1)
				else:
					foundPlayer.setTeam(1)
					foundPlayer.setTeam(2)
				Log(cmd, p.getName(), foundPlayer.getName(), reason)
	except:
		adf.Debug("Exception in PlayerResign()")


# Kill a player
def PlayerKill(cmd, args, p):
	try:
		if len(args) < 2:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name and a reason", p)
		else:
			i = 1
			reason = ""
			while i < len(args):
				reason += str(args[i] + " ")
				i += 1

			# Find a predefined reason
			reason = FindReason(reason)

			# Kill player
			foundPlayer = adf.FindPlayer(args[0])
			if foundPlayer == "none":
				adf.PersonalMessage("C1001Sorry, no matching player found for '" + args[0] + "'", p)
			elif foundPlayer == "more":
				adf.PersonalMessage("C1001Multiple players found that match '" + args[0] + "'", p)
			else:
				try:
					# Try to kill the player
					foundPlayer.getVehicle().setDamage(0.000000001)
					foundPlayer.setTimeToSpawn(0)
					AdminPM("C1001" + foundPlayer.getName() + " has been killed, " + reason, p)
					host.rcon_invoke('game.sayall "C1001KILLING PLAYER %s, %s"' % (foundPlayer.getName(), reason))
				except:
					# Couldn't kill the player (was probably in gunner-seat or so), mark him for death! (will die when he exits the vehicle)
					foundPlayer.die = 1
					foundPlayer.killreason = reason
					AdminPM("C1001" + foundPlayer.getName() + " has been marked for death", p)
					host.rcon_invoke('game.sayall "C1001Player %s has been marked for death!"' % foundPlayer.getName())
				Log(cmd, p.getName(), foundPlayer.getName(), reason)
	except:
		adf.Debug("Exception in PlayerKill")


# Take a punkbuster screenshot of a player
def PlayerScreenshot(cmd, args, p):
	try:
		if len(args) < 1:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name", p)
		else:
			foundPlayer = adf.FindPlayer(args[0])
			if foundPlayer == "none":
				adf.PersonalMessage("C1001Sorry, no matching player found for '" + args[0] + "'", p)
			elif foundPlayer == "more":
				adf.PersonalMessage("C1001Multiple players found that match '" + args[0] + "'", p)
			else:
				# Set the height and width
				host.rcon_invoke('PB_SV_SsWidth %s' % adfs.adm_pbSsWidth)
				host.rcon_invoke('PB_SV_SsHeight %s' % adfs.adm_pbSsHeight)

				# Take the screenshot
				host.rcon_invoke('pb_sv_getss "%s"' % foundPlayer.getName())
				AdminPM("C1001Screenshot taken of " + foundPlayer.getName(), p)
			Log(cmd, p.getName(), foundPlayer.getName(), "")
	except:
		adf.Debug("Exception in PlayerScreenshot()")


# Kick a player
def PlayerKick(cmd, args, p):
	try:
		if len(args) < 2:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name and a reason", p)
		else:
			i = 1
			reason = ""
			while i < len(args):
				reason += str(args[i] + " ")
				i += 1

			# Find a predefined reason
			reason = FindReason(reason)

			# Kick player
			foundPlayer = adf.FindPlayer(args[0])
			if foundPlayer == "none":
				adf.PersonalMessage("C1001Sorry, no matching player found for '" + args[0] + "'", p)
			elif foundPlayer == "more":
				adf.PersonalMessage("C1001Multiple players found that match '" + args[0] + "'", p)
			else:
				AdminPM("C1001" + foundPlayer.getName() + " has been kicked, " + reason, p)
				host.rcon_invoke('game.sayall "C1001KICKING PLAYER %s, %s"' % (foundPlayer.getName(), reason))
				if adfs.usePunkbuster == True:
					host.rcon_invoke('pb_sv_kick "%s" %i "%s"' % (foundPlayer.getName(), adfs.adm_kickTime, reason))
				else:
					host.rcon_invoke('admin.kickPlayer %d' % foundPlayer.index)
				Log(cmd, p.getName(), foundPlayer.getName(), reason)
	except:
		adf.Debug("Exception in PlayerKick()")


# Warn a player
def PlayerWarn(cmd, args, p):
	try:
		if len(args) < 2:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name and a reason", p)
		else:
			i = 1
			reason = ""
			while i < len(args):
				reason += str(args[i] + " ")
				i += 1

			# Find a predefined reason
			reason = FindReason(reason)

			# Warn player
			foundPlayer = adf.FindPlayer(args[0])
			if foundPlayer == "none":
				adf.PersonalMessage("C1001Sorry, no matching player found for '" + args[0] + "'", p)
			elif foundPlayer == "more":
				adf.PersonalMessage("C1001Multiple players found that match '" + args[0] + "'", p)
			else:
				AdminPM("C1001" + foundPlayer.getName() + " has been warned, " + reason, p)
				host.rcon_invoke('game.sayall "C1001WARNING PLAYER %s, %s"' % (foundPlayer.getName(), reason))	# Dispay warningmessage
				# AWARD_NAME_1220104_1           STOP DOING THAT
				host.sgl_sendMedalEvent(foundPlayer.index, 1220104, 1)
				Log(cmd, p.getName(), foundPlayer.getName(), reason)
	except:
		adf.Debug("Exception in PlayerWarn()")


# Ban a player
def PlayerBan(cmd, args, p):
	try:
		if len(args) < 2:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name and a reason", p)
		else:
			i = 1
			reason = ""
			while i < len(args):
				reason += str(args[i] + " ")
				i += 1

			# Find a predefined reason
			reason = FindReason(reason)

			# Ban player
			foundPlayer = adf.FindPlayer(args[0])
			if foundPlayer == "none":
				adf.PersonalMessage("C1001Sorry, no matching player found for '" + args[0] + "'", p)
			elif foundPlayer == "more":
				adf.PersonalMessage("C1001Multiple players found that match '" + args[0] + "'", p)
			else:
				AdminPM("C1001" + foundPlayer.getName() + " has been banned, " + reason, p)
				host.rcon_invoke('game.sayall "C1001BANNING PLAYER %s, %s"' % (foundPlayer.getName(), reason))	# Display banmessage

				if adfs.usePunkbuster == True:
					host.rcon_invoke('pb_sv_ban "%s" "%s"' % (foundPlayer.getName(), reason))
				else:
					host.rcon_invoke('admin.banPlayer %d' % foundPlayer.index)
				Log(cmd, p.getName(), foundPlayer.getName(), reason)
	except:
		adf.Debug("Exception in PlayerBan()")


# Temp ban a player
def PlayerBanTemp(cmd, args, p):
	try:
		if len(args) < 2:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name and a reason", p)
		else:
			i = 1
			reason = ""
			while i < len(args):
				reason += str(args[i] + " ")
				i += 1

			# Find a predefined reason
			reason = FindReason(reason)

			# Tempban the player
			foundPlayer = adf.FindPlayer(args[0])
			if foundPlayer == "none":
				adf.PersonalMessage("C1001Sorry, no matching player found for '" + args[0] + "'", p)
			elif foundPlayer == "more":
				adf.PersonalMessage("C1001Multiple players found that match '" + args[0] + "'", p)
			else:
				AdminPM("C1001" + foundPlayer.getName() + " has been tempbanned, " + reason, p)
				host.rcon_invoke('game.sayall "C1001TEMP BANNING PLAYER %s, %s"' % (foundPlayer.getName(), reason))	# Display banmessage

				if adfs.usePunkbuster == True:
					host.rcon_invoke('pb_sv_kick "%s" %i "%s"' % (foundPlayer.getName(), adfs.adm_banTime, reason))
				else:
					host.rcon_invoke('admin.kickPlayer %d' % foundPlayer.index)
				Log(cmd, p.getName(), foundPlayer.getName(), reason)
	except:
		adf.Debug("Exception in PlayerBanTemp()")


# Unban a player
def PlayerUnban(cmd, args, p):
	try:
		if adfs.usePunkbuster == False:
			adf.PersonalMessage("C1001Sorry, unbanning only works when Punkbuster is enabled", p)
		else:
			if len(args) < 1:
				adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name or reason", p)
			else:
				i = 1
				arg = ""
				while i < len(args):
					arg += str(args[i] + " ")
					i += 1

				# Retrieve banlist
				bl = host.rcon_invoke('pb_sv_banlist' + arg).strip().split("\n")

				# If there are multiple entries, tell the admin about it (there's always one line saying "end of banlist")
				if len(bl) > 1:
					adf.PersonalMessage("C1001There are " + str(len(bl)-1) + " entries that contain '" + arg + "'", p)
				
				# If there's only one entry, unban it
				else:
					# A ban entry looks like this: PunkBuster Server: 1   GGGGGGGGGGGGGGGGGGGGGGGGGG {0/-1} "PLAYERNAME" "1.2.3.4" THIS IS THE REASON
					b = bl.split()

					# b[2] contains the slotnumber
					# b[5] contains the playername
					host.rcon_invoke("pb_sv_unban " + b[2])
					host.rcon_invoke("pb_sv_updbanfile")
					
					AdminPM("C1001" + b[5] + " (slot #" + b[2] + ") has been unbanned", p)
					Log(cmd, p.getName(), "", arg)
	except:
		adf.Debug("Exception in PlayerUnban()")


# Retrieve the hash of a player
def PlayerHash(cmd, args, p):
	try:
		if len(args) < 1:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name", p)
		else:
			# Find the hash
			foundPlayer = adf.FindPlayer(args[0])
			if foundPlayer == "none":
				adf.PersonalMessage("C1001Sorry, no matching player found for '" + args[0] + "'", p)
			elif foundPlayer == "more":
				adf.PersonalMessage("C1001Multiple players found that match '" + args[0] + "'", p)
			else:
				h = adf.GetPlayerHash(foundPlayer.getName())
				adf.PersonalMessage("C1001The hash of " + foundPlayer.getName() + " is: " + h, p)
			Log(cmd, p.getName(), foundPlayer.getName(), "")
	except:
		adf.Debug("Exception in PlayerHash()")


# Fling a player
def PlayerFly(cmd, args, p):
	try:
		if len(args) < 2:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name and altitude (max: " + str(adfs.adm_maxAltitude) + ")", p)
		elif args[1].isdigit() == False or int(args[1]) > adfs.adm_maxAltitude:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name and altitude (max: " + str(adfs.adm_maxAltitude) + ")", p)
		else:
			foundPlayer = adf.FindPlayer(args[0])
			if foundPlayer == "none":
				adf.PersonalMessage("C1001Sorry, no matching player found for '" + args[0] + "'", p)
			elif foundPlayer == "more":
				adf.PersonalMessage("C1001Multiple players found that match '" + args[0] + "'", p)
			else:
				if foundPlayer.isAlive() == 0 or foundPlayer.isManDown() == 1:
					adf.PersonalMessage("C1001" + foundPlayer.getName() + " is currently dead or down", p)
				else:
					if int(args[1]) > adfs.adm_maxAltitude:
						adf.PersonalMessage("C1001Altitude is too high. The max altitude is: " + adfs.adm_maxAltitude, p)
					else:
						pos = foundPlayer.getVehicle().getPosition()

						# Let the player fly
						pos1_n = pos[1] + abs(int(args[1]))
						foundPlayer.getVehicle().setPosition(tuple([float(pos[0]), float( pos1_n ), float(pos[2])]))

						host.rcon_invoke('game.sayall "C1001FLINGING PLAYER %s"' % foundPlayer.getName())
						Log(cmd, p.getName(), foundPlayer.getName(), args[1])
	except:
		adf.Debug("Exception in PlayerFly()")


# Report a player
def PlayerReport(cmd, args, p):
	try:
		if len(args) < 1:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name and/or reason", p)
		else:
			foundPlayer = adf.FindPlayer(args[0])
			if foundPlayer == "none" or foundPlayer == "more":
				i = 0
			else:
				i = 1

			# Reason
			reason = ""
			while i < len(args):
				reason += str(args[i] + " ")
				i += 1

			# Find a predefined reason
			reason = FindReason(reason)

			if foundPlayer == "none" or foundPlayer == "more":
				AdminPM("C10013" + reason, p)
				Log(cmd, p.getName(), "", reason)
			else:
				AdminPM("C10013" + foundPlayer.getName() + " is reported, " + reason, p)
				Log(cmd, p.getName(), foundPlayer.getName(), reason)

			# Report player
			adf.PersonalMessage("C1001Your report has been sent to all in-game admins", p)
	except:
		adf.Debug("Exception in PlayerReport()")


# Teamwitch a player
def PlayerSwitch(cmd, args, p):
	try:
		if len(args) < 1 or (len(args) > 1 and args[1].lower() != "now"):
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a name (and optional, [now])", p)
		else:
			foundPlayer = adf.FindPlayer(args[0])
			if foundPlayer == "none":
				adf.PersonalMessage("C1001Sorry, no matching player found for '" + args[0] + "'", p)
			elif foundPlayer == "more":
				adf.PersonalMessage("C1001Multiple players found that match '" + args[0] + "'", p)
			else:
				# Check if playerhash is set
				try:
					if p.hash == "":
						pass
				except:
					adf.Debug("Playerhash-variable isn't set, player isn't inited, initiating now")
					PlayerInit(p)
				
				# Mark player for teamswitching (death/kill function will take care of that)
				foundPlayer.teamswitch = 1

				# If the player is in a vehicle, don't switch, unless "now" is specified
				if len(args) == 1 and foundPlayer.getDefaultVehicle() != foundPlayer.getVehicle() and foundPlayer.getVehicle().templateName != "MultiPlayerFreeCamera":
					host.rcon_invoke('game.sayall "C1001Player %s is in a vehicle (%s), so has been marked for teamswitching (on next death))"' % (foundPlayer.getName(), foundPlayer.getVehicle().templateName))
					Log(cmd, p.getName(), foundPlayer.getName(), "On next death")
				elif len(args) == 1 and foundPlayer.isAlive() == 1:
					host.rcon_invoke('game.sayall "C1001Player %s is not dead, so has been marked for teamswitching (on next death))"' % foundPlayer.getName())
					Log(cmd, p.getName(), foundPlayer.getName(), "On next death")
				else:
					# Try to kill the player, he needs to be switched right away
					try:
						if foundPlayer.getVehicle().templateName == "MultiPlayerFreeCamera":
							foundPlayer.teamswitch = 0
							if foundPlayer.getTeam() == 1:
								foundPlayer.setTeam(2)
							else:
								foundPlayer.setTeam(1)
							host.rcon_invoke('game.sayall "C1001Teamswitching %s"' % foundPlayer.getName())
						else:
							foundPlayer.getVehicle().setDamage(0.000000001)
						foundPlayer.setTimeToSpawn(0)
					except:
						host.rcon_invoke('game.sayall "C1001Player %s is in a vehicle (%s) and could not be killed, so has been marked for teamswitching (on next death or vehicle exit))"' % (foundPlayer.getName(), foundPlayer.getVehicle().templateName))
					Log(cmd, p.getName(), foundPlayer.getName(), "Instantly")
	except:
		adf.Debug("Exception in PlayerSwitch()")


# Teamswap everyone
def PlayerSwapTeams(cmd, args, p):
	try:
		adf.Debug("Teamswapping everone")
		AdminPM("C1001Teamwapping everyone", p)
		host.rcon_invoke('game.sayall "C1001Teamswapping everyone..."')
		for pl in bf2.playerManager.getPlayers():
			if pl.getTeam() == 1:
				pl.setTeam(2)
			else:
				pl.setTeam(1)
		Log(cmd, p.getName(), "", "")
	except:
		adf.Debug("Exception in PlayerTeamSwap()")


# Stop the gameserver
def ServerStop(cmd, args, p):
	try:
		if len(args) > 0:
			i = 0
			reason = ""
			while i < len(args):
				reason += str(args[i] + " ")
				i += 1

			# Find a predefined reason
			reason = FindReason(reason)

			# Stop the server
			Log(cmd, p.getName(), "", reason)
			AdminPM("C1001Server has been terminated", p)
			host.rcon_invoke('game.sayall "C1001SERVER IS TERMINATED, PLEASE RECONNECT"')
			host.rcon_invoke("quit")
		else:
			adf.PersonalMessage("C1001Incorrect usage of '" + cmd + "'. Please specify a reason", p)
	except:
		adf.Debug("Exception in ServerStop")


# Turn voting on/off
def ServerVote(cmd, args, p):
	try:
		if len(args) > 0 and (args[0].lower() != "on" and args[0].lower() != "off"):
			adf.PersonalMessage("C1001Please specify a (correct) parameter (on/off)", p)
		else:
			if len(args) == 0:
				v = str(host.rcon_invoke('sv.votingEnabled').strip())
				if v == "1":
					v = "enabled"
				else:
					v = "disabled"
				adf.PersonalMessage("C1001Voting is " + v, p)
			else:
				if args[0].lower() == "on":
					host.rcon_invoke('sv.votingEnabled 1')
					AdminPM("C1001Voting has been enabled", p)
				else:
					host.rcon_invoke('sv.votingEnabled 0')
					AdminPM("C1001Voting has been disabled", p)
			Log(cmd, p.getName(), "", args[0].lower())
	except:
		adf.Debug("Exception in ServerVote()")


# Reinitialize the settings file
def PythonInit(cmd, args, p):
	try:
		import ad_framework_settings as adfsreload
		#reload(ad_framework_settings)
		reload(adfsreload)
		adfs.adm_adminHashes       = adfsreload.adm_adminHashes
		adfs.adm_adminPowerLevels  = adfsreload.adm_adminPowerLevels
		AdminPM("C1001Adminhashes and -powerlevels have been reloaded", p)
		Log(cmd, p.getName(), "", "")
	except:
		adf.Debug("Exception in PythonInit")


# Enable/Disable smart balancing
def PythonBalance(cmd, args, p):
	try:
		if len(args) > 0 and (args[0].lower() != "on" and args[0].lower() != "off"):
			adf.PersonalMessage("C1001Please specify a (correct) parameter (on/off) and optional (difference)", p)
		else:
			if len(args) == 0:
				if adfs.smb_enabled == True:
					b = "enabled"
				else:
					b = "disabled"
				adf.PersonalMessage("C1001AD Smart Balance is " + b + ", with an allowed balance difference of " + str(adfs.smb_difference), p)
			else:
				i = adfs.smb_difference
				if len(args) > 1:
					try:
						i = int(args[1])
					except:
						pass
					adfs.smb_difference = i
				
				if args[0].lower() == "on":
					adfs.smb_enabled = True
					AdminPM("C1001AD Smart Balance has been enabled, with an allowed balance difference of " + str(i), p)
				else:
					adfs.smb_enabled = False
					AdminPM("C1001AD Smart Balance has been disabled, the allowed balance difference is " + str(i), p)
				Log(cmd, p.getName(), "", args[0].lower())
	except:
		adf.Debug("Exception in PythonBalance")


# Enable/Disable debugging
def PythonDebug(cmd, args, p):
	try:
		if len(args) < 1 or (args[0].lower() != "on" and args[0].lower() != "off"):
			adf.PersonalMessage("C1001Please specify a (correct) parameter (on/off)", p)
		else:
			if args[0].lower() == "on":
				adfs.debugging = True
				AdminPM("C1001AD Framework debugging has been enabled", p)
			else:
				adfs.debugging = False
				AdminPM("C1001AD Framework debugging has been disabled", p)
			Log(cmd, p.getName(), "", args[0].lower())
	except:
		adf.Debug("Exception in PythonDebug")


#
# ======================= EVENT HANDLERS =======================
#


# When a player connects, set our custom variables
def onPlayerConnect(p):
	try:
		PlayerInit(p)
	except:
		adf.Debug("Exception in onPlayerConnect")


# When a player exits a vehicle and has been marked for killing (for whatever purpose: !kill or !switch), do it!
def onExitVehicle(p, vehicle):
	try:
		# Check if playervars are set
		try:
			if p.die == 0:
				pass
		except:
			adf.Debug("Playervariables aren't set, initiating now")
			PlayerInit(p)
		
		# If the player is marked for death!
		if p.die == 1:
			try:
				# Try to kill the player
				p.getVehicle().setDamage(0.000000001)
				p.setTimeToSpawn(0)
				host.rcon_invoke('game.sayall "C1001KILLING PLAYER %s, %s"' % (p.getName(), p.killreason))
				p.die = 0
				p.killreason = ""
			except:
				PlayerInit(p)
				return

		# If player is marked for teamswitch ("now")
		if p.teamswitch == 2:
			try:
				# Try to kill the player
				foundPlayer.getVehicle().setDamage(0.000000001)
				foundPlayer.setTimeToSpawn(0)
				TeamSwitch(p)
			except:
				return
	except:
		adf.Debug("Exception in onExitVehicle")
		PlayerInit(p)


# When a player gets killed, check if he needs to teamswitch
def onPlayerKilled(p, attacker, weapon, assists, object):
	try:
		p.die = 0
		p.killreason = ""
		TeamSwitch(p)
	except:
		adf.Debug("Exception in onPlayerKilled")


# When a player dies, check if he needs to teamswitch
def onPlayerDeath(p, vehicle):
	try:
		p.die = 0
		p.killreason = ""
		TeamSwitch(p)
	except:
		adf.Debug("Exception in onPlayerDeath")


# Whenever a chat message is sent on any channel, check it for commands; if
# there are any, check if it's an admin saying it and if so, execute the command
def onChatMessage(playerID, msgText, channel, flags):
	global adminCommands, isDedicated

	try:
		if len(msgText) > 1 and playerID != -1:
			# Remove unnecessary stuff that precedes the chat
			msgText = msgText.replace("HUD_TEXT_CHAT_TEAM", "")
			msgText = msgText.replace("HUD_TEXT_CHAT_SQUAD", "")
			msgText = msgText.replace("HUD_TEXT_CHAT_DEADPREFIX", "")
			msgText = msgText.replace("HUD_CHAT_DEADPREFIX", "")

			# Remove the space and/or star that preceed the message in global chat
			if msgText.startswith("*"):
				msgText = msgText.replace("*", "", 1)
			if msgText.startswith(" "):
				msgText = msgText.replace(" ", "", 1)

			# If the entered text doesn't start with a admin command symbol, cancel further execution
			if not msgText.startswith(adfs.adm_commandSymbol):
				return False

			# Split the entered text and remove the admin command symbol
			command = msgText.split(" ")[0].replace(adfs.adm_commandSymbol, "", 1)

			# If the command doesn't exist in the adminCommands dictionary, cancel further execution
			if not adminCommands.has_key(command):
				adf.Debug("The entered text was not a command")
				return False

			# Set the P(layer) object
			p = bf2.playerManager.getPlayerByIndex(playerID)

			# If this is a dedicated server and the command is NOT an "open" command (like !r)
			if isDedicated and adfs.adm_adminPowerLevels[command] != 777:
				adf.Debug("Server is dedicated, proceeding with hash checks")

				# Check if playervars are set
				try:
					if p.teamswitch == "":
						pass
				except:
					adf.Debug("Playervariables aren't set, initiating now")
					PlayerInit(p)

				# If p.hash is not set, set it, so we don't have to search the playerlist for him the next time
				if p.hash == "":
					adf.Debug("Player has no hash set, finding it now")
					tHash = adf.GetPlayerHash(p.getName())
					if tHash == False:
						return False
					else:
						adf.Debug("Hash found, setting it now")
						p.hash = tHash

				# A hash is a key in the adm_adminHashes array, check if the found hash exists
				if not adfs.adm_adminHashes.has_key(p.hash):
					adf.Debug("The player's hash doesn't match an admin's hash")
					return False

				adf.Debug("Hash found!")

			# Server is not dedicated, admin commands are available to all human players
			else:
				adf.Debug("Server is not dedicated (or an 'open' command was used), not performing hash check!")

			# If the server is dedicated and the command is NOT an "open" command, check if the admin has the rights to execute the command
			if adfs.adm_adminPowerLevels[command] != 777:
				if isDedicated and adfs.adm_adminHashes[p.hash] > adfs.adm_adminPowerLevels[command]:
					adf.Debug("Admin doesn't have enough rights to execute this command!")
					adf.PersonalMessage("C1001You do not have sufficient rights to execute this command", p)
					return False

			# Execute the command and pass the arguments along as an array (without the admin command symbol and the command)
			adf.Debug("Executing command now...")
			args = msgText.replace(adfs.adm_commandSymbol + command, "", 1).strip().split()
			adminCommands[command](command, args, p)
			adf.Debug("Done executing")
	except:
		adf.Debug("Exception in onChatMessage() :: Could be caused by a function that is called, that has no exception handler")


#
# ======================= CUSTOM FUNCTIONS =======================
#


# Init the player
def PlayerInit(p):
	try:
		p.teamswitch = 0
		p.die = 0
		p.killreason = ""
		p.hash = ""
	except:
		adf.Debug("Exception in PlayerInit()")


# Log admincommands
def Log(cmd, an, pn, text):
	global loggingFile, loggingFileName

	try:
		if adfs.adm_logging == True:
			loggingFile = open(loggingFileName, "a")

			d = time.strftime("%Y-%m-%d %H:%M:%S")

			message = ""
			while len(cmd) < 15:
				cmd += " "

			if pn == "":
				loggingFile.write(d + ": " + cmd.upper() + " performed by '" + an + "': " + text + "\n")
			else:
				loggingFile.write(d + ": " + cmd.upper() + " performed by '" + an + "' on '" + pn + "': " + text + "\n")

			loggingFile.close()
	except:
		adf.Debug("Exception in Log()")


# Teamswitch a player
def TeamSwitch(p):
	try:
		if p.teamswitch == 1:
			host.rcon_invoke('game.sayall "C1001Teamswitching player %s"' % p.getName())
			p.teamswitch = 0
			if p.getTeam() == 1:
				p.setTeam(2)
			else:
				p.setTeam(1)
	except:
		adf.Debug("Variable teamswitch not set, setting it now")
		PlayerInit(p)


# This function constructs the maplist
def ConstructMaplist():
	global mapList

	try:
		mapList = []
		map_list = host.rcon_invoke("maplist.list").strip().split("\n")

		for map in map_list:
			# Get rid of the " around the mapname
			map = map.replace("\"", "")

			# Split the map, to extract the ID
			map_splitted = map.split(": ")
			map_id = map_splitted[0]

			# Save the map information to the maplist
			map_splitted = map_splitted[1].split(" ")

			# Translate mapsize
			if map_splitted[2] == "16":
				map_splitted[2] = "inf"
			elif map_splitted[2] == "32":
				map_splitted[2] = "alt"
			elif map_splitted[2] == "64":
				map_splitted[2] = "std"

			# Add map to array
			mapDetails = str(map_id) + "|" + map_splitted[0] + "|" + map_splitted[1] + "|" + map_splitted[2]
			mapList[len(mapList):] = [mapDetails]
	except:
		adf.Debug("Exception in ConstructMaplist")


# Format mapproperties
def FormatMapProperties(value):
	containsGamemode = False

	# If the string contains the gamemode, replace it with 'words'
	if value.lower().find("gpm_") != -1:
		containsGamemode = True
		value = value.replace("gpm_cnc", "Command And Control")
		value = value.replace("gpm_coop", "COOP")
		value = value.replace("gpm_cq", "AAS")
		value = value.replace("gpm_insurgency", "Insurgency")
		value = value.replace("gpm_skirmish", "Skirmish")
		value = value.replace("gpm_vehicles", "Vehicle Warfare")

	value = value.replace("_", " ")

	# Only "title()" the string when no gamemode is found (otherwise "AAS" would become "Aas", which is lame)
	if containsGamemode == False:
		value = value.title()

	return value


# Search for a custom reason
def FindReason(reason):
	try:
		if adfs.adm_reasons.has_key(reason.lower().strip()):
			adf.Debug("Reason found!")
			reason = adfs.adm_reasons[reason.lower().strip()]
		else:
			adf.Debug("Reason does not exist")
		return reason
	except:
		adf.Debug("Exception in FindReason()")


# Send a personal message to all in-game admins. Argument "p" represents the admin that's causing it
def AdminPM(msg, p):
	# Loop through all players, find all admins
	try:
		msg += " [" + p.getName() + "]"

		# If we're running a dedicated server, send a personal message, otherwise send a global message
		if isDedicated == True:
			for p in bf2.playerManager.getPlayers():
				# Check if the playerhash is set
				try:
					if p.hash == "":
						pass
				except:
					adf.Debug("Playerhash-variable isn't set, player isn't inited, initiating now")
					PlayerInit(p)

				# Get the hash
				if p.hash == "":
					tHash = adf.GetPlayerHash(p.getName())
					if tHash == False:
						adf.Debug("Hash not found. Not sending a PM to " + p.getName())
					else:
						adf.Debug("Hash found, setting it now")
						p.hash = tHash

				# If the hash is set and it appears in the adminHashes dictionary, we have found ourselves an administrator
				if p.hash != "" and adfs.adm_adminHashes.has_key(p.hash):
					adf.PersonalMessage(msg, p)		
		else:
			host.rcon_invoke('game.sayall "%s"' % msg)
	except:
		adf.Debug("Exception in AdminPM()")
