# -*- coding: iso-8859-15 -*-
#
# AD SMART BALANCE
#
#
# ******************************************
#    DO NOT MAKE ANY CHANGES TO THIS FILE
# ******************************************
#


import host, bf2, sys, default, new, string, re, bf2.Timer, random
import ad_framework_settings as adfs
import ad_framework as adf

# Timer for pregame balancing
preGameBalanceTimer = ""

# Variable to store if the teams have been swapped or not
teamSwapped = 0


# Initialise the smart-balance module
def init():
	global preGameBalanceTimer

	adf.Debug("Initializing AD Smart Balance")

	# Register handlers
	host.registerHandler('PlayerConnect', onPlayerConnect)
	host.registerHandler('PlayerDeath', onPlayerDeath)
	host.registerHandler('PlayerChangeTeams', onPlayerChangeTeams)
	host.registerHandler('PlayerSpawn', onPlayerSpawn)
	host.registerGameStatusHandler(onGameStatusChanged)


# When a player changes teams, check if the balance is okay
def onPlayerChangeTeams(p, humanHasSpawned):
	SmartBalance(p)


# When a player connects, check if the balance is okay
def onPlayerConnect(p):
	SmartBalance(p)


# When a player dies, check if the balance is okay
def onPlayerDeath(p, vehicle):
	SmartBalance(p)


# When a player spawns, the timer for the pregame smartbalance should be destroyed
def onPlayerSpawn(p, vehicle):
	global preGameBalanceTimer

	if preGameBalanceTimer:
		adf.Debug("Timer is destroyed!")
		host.timer_destroy(preGameBalanceTimer)
		preGameBalanceTimer = ""


# This function checks if a player should be switched or not
def SmartBalance(p):
	adf.Debug("Entered SmartBalance()")

	# If smart balancing is enabled...
	if adfs.smb_enabled == True:
		# Team playercounters
		team1 = 0
		team2 = 0

		# If the player is not alive, not SL, not CMDR and not a bot, continue...
		if p.isAlive() == 0 and p.isSquadLeader() == 0 and p.isCommander() == 0 and p.isAIPlayer() == 0:
			# If the player is wearing a specified clantag, don't do a thing, otherwise, see if switching is needed
			clanTagFound = False
			for ct in adfs.smb_excludeList:
				if p.getName().find(ct) != -1:
					adf.Debug("Clantag found, doing nothing")
					clanTagFound = True
					break

			adf.Debug("Clantag not found")
			if clanTagFound == False:
				adf.Debug("Checking if player " + p.getName() + " needs to be switched")
				team1 = bf2.playerManager.getNumberOfPlayersInTeam(1)
				team2 = bf2.playerManager.getNumberOfPlayersInTeam(2)

				# Set the team to switch players from
				if team2 > (team1 + adfs.smb_difference):
					adf.Debug("Switching player " + p.getName() + " to team 1")
					p.setTeam(1)
				elif team1 > (team2 + adfs.smb_difference):
					p.setTeam(2)
					adf.Debug("Switching player " + p.getName() + " to team 2")
	else:
		adf.Debug("Smart balancing is disabled, so not doing anything")


# Before the game starts (when players can spawn, that is), also do smart balancing
def onGameStatusChanged(status):
	global preGameBalanceTimer, teamSwapped

	if status == bf2.GameStatus.Playing:
		# Kill the timer, if it exists
		adf.Debug("Timer is destroyed in onGameStatusChanged!")
		host.timer_destroy(preGameBalanceTimer)
		preGameBalanceTimer = ""

		# Set timer to do pregame balancing
		adf.Debug("Timer started...")
		preGameBalanceTimer = bf2.Timer(PreGameSmartBalance, 3, 1)
		preGameBalanceTimer.setRecurring(3)

		# Raise with 1 (this is because the server enters the state "playing" twice. Without this check, you'd get teamswapped twice as well.
		teamSwapped += 1

		# If smart balancing is enabled, and the teams should be swapped on round start, and it hasn't done so before, do iiit.
		if adfs.smb_enabled == True and adfs.smb_swapTeamsOnStart == True and teamSwapped == 2:
			host.rcon_invoke('game.sayall "C1001Teamswapping everyone..."')
			for p in bf2.playerManager.getPlayers():
				adf.Debug("Teamswapping player " + p.getName() + " (because of teamswap on roundstart)")
				if p.getTeam() == 1:
					p.setTeam(2)
				else:
					p.setTeam(1)

			# Set teamSwapped to True, so it doesn't teamswap you again on the same round, darn it.
			teamSwapped = 0


# Function that selects a random player and throws it into SmartBalance()
def PreGameSmartBalance(data):
	adf.Debug("Entered PreGameSmartBalance()")

	# If smart balancing is enabled...
	if adfs.smb_enabled == True:
		team1 = bf2.playerManager.getNumberOfPlayersInTeam(1)
		team2 = bf2.playerManager.getNumberOfPlayersInTeam(2)

		adf.Debug("Team1: %s. Team 2: %s." % (str(team1), str(team2)))

		if team1 > (team2 + adfs.smb_difference):
			adf.Debug("Team 1 has more players than team 2")
			teamToCheck = 1
		elif team2 > (team1 + adfs.smb_difference):
			adf.Debug("Team 2 has more players than team 1")
			teamToCheck = 2
		else:
			return False

		playerToCheck = random.randint(1, bf2.playerManager.getNumberOfPlayersInTeam(teamToCheck))
		adf.Debug("Random player selected to perform smart balance on: " + str(playerToCheck) + " (index)")

		i = 1
		for p in bf2.playerManager.getPlayers():
			if p.getTeam() == teamToCheck:
				if i == playerToCheck:
					SmartBalance(p)
				i += 1
